@@ -31,6 +31,9 @@ public:
 	int setsize;
 	int matchsize;
 	int closedelay;
+	int motifcaption;
+	int autoflip;
+	int autoflipframe;
 
 	/* sound */
 	int sound;
