@@ -29,8 +29,9 @@ View::View(Renderer &r, Config &cfg, Game &gm)
 	  lblCredits1(true), lblCredits2(true), noGameYet(true),
 	  state(VS_IDLE),
 	  game(gm), quitReceived(false),
-	  lblScore(true), lblTime(true), lblErrors(true),
+	  lblInfoLeft(true), lblTime(true), lblInfoRight(true),
 	  mcx(-1), mcy(-1),
+	  captionKeyPressed(0),
 	  fpsCycles(0), fpsStart(0), fps(0)
 {
 	mixer.open(config.channels, config.audiobuffersize);
@@ -78,7 +79,8 @@ void View::init(string t, uint f)
 	/* set label stuff */
 	lblCredits1.setText(theme.fSmall, "http://lgames.sf.net");
 	lblCredits2.setText(theme.fSmall, string("v")+PACKAGE_VERSION);
-	lblRestart.setText(theme.fNormal, _("Press R to restart (or ESC for menu)."));
+	lblRestart.setText(theme.fNormal, _("Press R or right mouse button to restart (or ESC for menu)."));
+	btnMenu.set(_("Menu"), theme.fNormal, theme.fNormalHighlighted, 0, 0, ALIGN_X_LEFT | ALIGN_Y_TOP);
 
 	/* create render images and positions */
 	imgBackground.create(sw,sh);
@@ -88,7 +90,7 @@ void View::init(string t, uint f)
 	if (noGameYet) {
 		changeWallpaper();
 		game.init(renderer.rx2sx(1), renderer.ry2sy(0.90),
-				GM_SOLO, GM_HUGE, 2, config.fullscreen, theme.numCards*2);
+				GM_SOLO, GM_HUGE, 2, config.fullscreen, theme.numMotifs);
 	} else
 		startGame();
 	cxoff = renderer.rx2sx(0.00);
@@ -110,6 +112,7 @@ void View::run()
 	string str;
 	int button = 0; /* pressed button */
 	int buttonX = 0, buttonY = 0; /* position if pressed */
+	int oldMCX = -1, oldMCY = -1;
 
 	state = VS_IDLE;
 
@@ -151,32 +154,62 @@ void View::run()
 					/* fake mouse button pressed */
 					if (!menuActive && state == VS_IDLE) {
 						button = SDL_BUTTON_LEFT;
-						buttonX = buttonY = -1;
+						SDL_GetMouseState(&buttonX, &buttonY);
 					}
 					break;
 				default:
 					break;
 				}
 			}
+
+			/* handle menu button to switch menu on/off */
+			if (btnMenu.handleEvent(ev))
+				if (!noGameYet)
+					menuActive = !menuActive;
+
 			if (menuActive)
 				handleMenuEvent(ev);
-			else if (ev.type == SDL_MOUSEBUTTONUP && state == VS_IDLE) {
-				button = ev.button.button;
-				buttonX = ev.button.x;
-				buttonY = ev.button.y;
+			else if (ev.type == SDL_MOUSEBUTTONUP) {
+				if (ev.button.button == SDL_BUTTON_RIGHT && game.gameover)
+					startGame();
+				else if (state == VS_IDLE) {
+					button = ev.button.button;
+					buttonX = ev.button.x;
+					buttonY = ev.button.y;
+				}
 			}
 		}
 
+		/* get passed time */
+		ms = ticks.get();
+
 		/* get input state */
+		oldMCX = mcx;
+		oldMCY = mcy;
 		SDL_GetMouseState(&mcx, &mcy);
 		if (state == VS_IDLE && game.getCurrentPlayer().isCPU()) {
 			game.getNextCPUClick(button, buttonX, buttonY);
 			buttonX += cxoff;
 			buttonY += cyoff;
 		}
+		/* check auto flip */
+		if (abs(mcx - oldMCX) > 3 || abs(mcy - oldMCY) > 3)
+			autoFlipDelay.reset();
+		if (config.autoflip && !menuActive && state == VS_IDLE &&
+				game.isOnFlippableCard(mcx - cxoff, mcy - cyoff)) {
+			if (autoFlipDelay.update(ms)) {
+				button = SDL_BUTTON_LEFT;
+				buttonX = mcx;
+				buttonY = mcy;
+			}
+		}
 
-		/* get passed time */
-		ms = ticks.get();
+		/* get key state */
+		const Uint8 *keystate = SDL_GetKeyboardState(NULL);
+		if (keystate[SDL_SCANCODE_LSHIFT] || keystate[SDL_SCANCODE_RSHIFT])
+			captionKeyPressed = 1;
+		else
+			captionKeyPressed = 0;
 
 		/* update animations and particles */
 		for (auto it = begin(sprites); it != end(sprites); ++it) {
@@ -215,7 +248,7 @@ void View::run()
 			renderPlayerInfo();
 			mixer.play(theme.sRemove);
 		}
-		if (flags & GF_ERRORSCHANGED)
+		if (flags & GF_MISTAKESCHANGED)
 			if (game.numPlayers == 1)
 				renderPlayerInfo();
 		if (flags & GF_CARDOPENED) {
@@ -244,7 +277,7 @@ void View::run()
 				if (game.curPlayer == 1)
 					dx = renderer.rx2sx(1) - c.w;
 				a = new FadeAnimation(
-					theme.cards[c.id], cxoff+c.x, cyoff+c.y,
+					theme.motifs[c.id].getTexture(), cxoff+c.x, cyoff+c.y,
 							dx, renderer.ry2sy(1), c.w, c.h,
 							ANIM_FADEDURATION);
 				sprites.push_back(unique_ptr<FadeAnimation>(a));
@@ -296,14 +329,29 @@ void View::render()
 		if (c.removed || skipAnimatedCard(i))
 			continue;
 		if (c.open || noGameYet)
-			theme.cards[c.id].copy(cxoff + c.x,cyoff + c.y,c.w,c.h);
+			theme.motifs[c.id].getTexture().copy(cxoff + c.x,cyoff + c.y,c.w,c.h);
 		else if (!menuActive && state == VS_IDLE &&
 				!game.getCurrentPlayer().isCPU() &&
-				c.hasFocus(mcx - cxoff, mcy - cyoff))
+				c.hasFocus(mcx - cxoff, mcy - cyoff)) {
 			theme.cardFocus.copy(cxoff + c.x,cyoff + c.y,c.w,c.h);
-		else
+			/* draw flip frame if autoflip is running */
+			if (autoFlipDelay.isRunning())
+				renderFlipFrame(i);
+		} else
 			theme.cardBack.copy(cxoff + c.x,cyoff + c.y,c.w,c.h);
 
+		/* motif caption */
+		if (!menuActive && state == VS_IDLE &&
+				(config.motifcaption == 2 ||
+				(config.motifcaption == 1 && captionKeyPressed)) &&
+					!game.getCurrentPlayer().isCPU() &&
+					c.open == true &&
+					c.hasFocus(mcx - cxoff, mcy - cyoff))
+			theme.motifs[c.id].getLabel().copy(
+					cxoff + c.x + c.w/2,
+					cyoff + c.y + c.h,
+					ALIGN_X_CENTER | ALIGN_Y_BOTTOM);
+
 		/* DEBUG
 		theme.fSmall.setAlign(ALIGN_X_RIGHT | ALIGN_Y_BOTTOM);
 		theme.fSmall.write(cxoff + c.x+c.w, cyoff + c.y+c.h,
@@ -312,9 +360,9 @@ void View::render()
 
 	/* labels */
 	lblTime.copy(renderer.rx2sx(0.5),renderer.ry2sy(0.025));
-	lblScore.copy(renderer.rx2sx(0.01),renderer.ry2sy(0.975),
+	lblInfoLeft.copy(renderer.rx2sx(0.01),renderer.ry2sy(0.975),
 			ALIGN_X_LEFT | ALIGN_Y_CENTER);
-	lblErrors.copy(renderer.rx2sx(0.99),renderer.ry2sy(0.975),
+	lblInfoRight.copy(renderer.rx2sx(0.99),renderer.ry2sy(0.975),
 			ALIGN_X_RIGHT | ALIGN_Y_CENTER);
 
 	if (game.gameover && !menuActive) {
@@ -324,6 +372,7 @@ void View::render()
 			lblResult.copy(renderer.rx2sx(0.5),renderer.ry2sy(0.75),
 					ALIGN_X_CENTER | ALIGN_Y_CENTER);
 	}
+	btnMenu.render();
 
 	/* sprites */
 	for (auto& s : sprites)
@@ -400,25 +449,27 @@ bool View::showInfo(const vector<string> &text, int type)
 
 void View::createMenus()
 {
-	Menu *mNewGame, *mOptions, *mAudio, *mGraphics;
-	const char *diffNames[] = {_("Small"),_("Medium"),_("Large"),_("Huge") } ;
+	Menu *mNewGame, *mAudio, *mGraphics;
+	const char *diffNames[] = {_("Tiny"),_("Small"),_("Medium"),_("Large"),_("Huge") } ;
 	const char *fpsLimitNames[] = {_("No Limit"),_("200 FPS"),_("100 FPS") } ;
 	const int bufSizes[] = { 256, 512, 1024, 2048, 4096 };
 	const int channelNums[] = { 8, 16, 32 };
-	const char *modeNames[] = {_("Solo"), _("Vs CPU"), _("Vs Human") };
+	const char *modeNames[] = {_("Solo"), _("Vs CPU"), _("Vs Human"), _("Survivor")};
+	const char *captionModeNames[] = {_("Off"),_("On Shift"),_("Always")};
+	const char *autoflipOptions[] = { _("Off"), _("Fast"), _("Slow") };
 
 	/* XXX too lazy to set fonts for each and every item...
 	 * use static pointers instead */
 	MenuItem::fNormal = &theme.fMenuNormal;
 	MenuItem::fFocus = &theme.fMenuFocus;
 	MenuItem::fTooltip = &theme.fSmall;
-	MenuItem::tooltipWidth = 0.3 * theme.menuBackground.getWidth();
+	MenuItem::tooltipWidth = 0.4 * renderer.w;
 
 	rootMenu.reset(); /* delete any old menu ... */
 
 	rootMenu = unique_ptr<Menu>(new Menu(theme)); /* .. or is assigning a new object doing it? */
 	mNewGame = new Menu(theme);
-	mOptions = new Menu(theme);
+	//mOptions = new Menu(theme);
 	mAudio = new Menu(theme);
 	mGraphics = new Menu(theme);
 	graphicsMenu = mGraphics; /* needed to return after mode/theme change */
@@ -426,17 +477,23 @@ void View::createMenus()
 	mNewGame->add(new MenuItem(_("Start Game"),"",AID_STARTGAME));
 	mNewGame->add(new MenuItemSep());
 	mNewGame->add(new MenuItemList(_("Mode"),
-			_("In fullscreen: Small=6x4, Medium=8x4, Large=10x5, Huge=12x6.\nNote that it's slightly different in window mode to match the different ratio."),
-			AID_NONE,config.gamemode,modeNames,3));
+			_("You can play alone or against a human or CPU opponent.\n\nIn Survivor you play endlessly with increasing set sizes until you made too many mistakes. In Survivor you always match pairs so the match size option below is ignored."),
+			AID_NONE,config.gamemode,modeNames,4));
 	mNewGame->add(new MenuItemList(_("Set Size"),
-			_("In fullscreen: Small=6x4, Medium=8x4, Large=10x5, Huge=12x6.\nNote that it's slightly different in window mode to match the different ratio."),
-			AID_NONE,config.setsize,diffNames,4));
+			_("Tiny=4x4, Small=6x4, Medium=8x4, Large=10x5, Huge=12x6.\nNote, that it's slightly different in window mode to match the different ratio."),
+			AID_NONE,config.setsize,diffNames,5));
 	mNewGame->add(new MenuItemRange(_("Match Size"),
 			"2 = Pairs, 3 = Triplets, 4 = Quadruplets\nNote that you always have to turn over that many cards regardless of a mismatch.",
 			AID_NONE,config.matchsize,2,4,1));
 	mNewGame->add(new MenuItemRange(_("Close Delay"),
 			"Time in seconds until opened cards are turned over again.",
 			AID_NONE,config.closedelay,1,5,1));
+	mNewGame->add(new MenuItemList(_("Captions"),
+			_("Display caption of open card if mouse pointer is on it. With 'On Shift' a shift key must additionally be pressed."),
+			AID_NONE,config.motifcaption,captionModeNames,3));
+	mNewGame->add(new MenuItemList(_("Auto Flip"),
+			_("Automatically flip over a card when the mouse pointer remains still on it."),
+			AID_AUTOFLIP,config.autoflip,autoflipOptions,3));
 	mNewGame->add(new MenuItemSep());
 /*	mNewGame->add(new MenuItemRange(_("Players"),
 			_("Number and names of players. Players alternate whenever a life is lost."),
@@ -463,11 +520,11 @@ void View::createMenus()
 			"Maximum number of frames per second.",
 			AID_NONE,config.fps,fpsLimitNames,3));
 	mGraphics->add(new MenuItemSep());
-	mGraphics->add(new MenuItemBack(mOptions));
+	mGraphics->add(new MenuItemBack(rootMenu.get()));
 
 	mAudio->add(new MenuItemSwitch(_("Sound"),"",AID_SOUND,config.sound));
 	mAudio->add(new MenuItemRange(_("Volume"),"",AID_VOLUME,config.volume,0,100,10));
-	mAudio->add(new MenuItemSwitch(_("Speech"),"",AID_NONE,config.speech));
+	//mAudio->add(new MenuItemSwitch(_("Speech"),"",AID_NONE,config.speech));
 	mAudio->add(new MenuItemSep());
 	mAudio->add(new MenuItemIntList(_("Buffer Size"),
 			_("Reduce buffer size if you experience sound delays. Might have more CPU impact though. (not applied yet)"),
@@ -478,17 +535,19 @@ void View::createMenus()
 	mAudio->add(new MenuItem(_("Apply Size&Channels"),
 			_("Apply above settings"),AID_APPLYAUDIO));
 	mAudio->add(new MenuItemSep());
-	mAudio->add(new MenuItemBack(mOptions));
+	mAudio->add(new MenuItemBack(rootMenu.get()));
 
-	mOptions->add(new MenuItemSub(_("Graphics"),mGraphics));
+	/* mOptions->add(new MenuItemSub(_("Graphics"),mGraphics));
 	mOptions->add(new MenuItemSub(_("Audio"),mAudio));
-	mOptions->add(new MenuItemBack(rootMenu.get()));
+	mOptions->add(new MenuItemBack(rootMenu.get())); */
 
 	rootMenu->add(new MenuItemSub(_("New Game"), mNewGame));
 	rootMenu->add(new MenuItemSep());
-	rootMenu->add(new MenuItemSub(_("Settings"), mOptions));
+	rootMenu->add(new MenuItemSub(_("Graphics"),mGraphics));
+	rootMenu->add(new MenuItemSub(_("Audio"),mAudio));
+	//rootMenu->add(new MenuItemSub(_("Settings"), mOptions));
 	//rootMenu->add(new MenuItem(_("Help"), "", AID_HELP));
-	//rootMenu->add(new MenuItemSep());
+	rootMenu->add(new MenuItemSep());
 	rootMenu->add(new MenuItem(_("Quit"), "", AID_QUIT));
 
 	rootMenu->adjust();
@@ -638,6 +697,9 @@ void View::handleMenuEvent(SDL_Event &ev)
 			menuActive = false;
 			waitForInputRelease();
 			break;
+		case AID_AUTOFLIP:
+			autoFlipDelay.init(config.autoflip*500);
+			break;
 		}
 	}
 }
@@ -647,11 +709,12 @@ void View::startGame()
 	changeWallpaper();
 	game.init(renderer.rx2sx(1), renderer.ry2sy(0.90),
 			config.gamemode, config.setsize, config.matchsize,
-			config.fullscreen, theme.numCards*config.matchsize);
+			config.fullscreen, theme.numMotifs);
 	noGameYet = false;
 	lblTime.setText(theme.fNormal, _("Time: 0:00"));
 	renderPlayerInfo();
 	shadowOffset = game.cgap / 2;
+	autoFlipDelay.init(config.autoflip*500);
 }
 
 void View::changeWallpaper()
@@ -673,11 +736,11 @@ void View::startTurningAnimation(uint cid)
 			c.w, c.h, ANIM_TURNDURATION);
 	sprites.push_back(unique_ptr<TurnAnimation>(ta));
 	if (c.open)
-		ta = new TurnAnimation(theme.cardBack, theme.cards[c.id],
+		ta = new TurnAnimation(theme.cardBack, theme.motifs[c.id].getTexture(),
 				cxoff+c.x+c.w/2, cyoff+c.y+c.h/2, c.w, c.h,
 				ANIM_TURNDURATION);
 	else
-		ta = new TurnAnimation(theme.cards[c.id], theme.cardBack,
+		ta = new TurnAnimation(theme.motifs[c.id].getTexture(), theme.cardBack,
 				cxoff+c.x+c.w/2, cyoff+c.y+c.h/2, c.w, c.h,
 				ANIM_TURNDURATION);
 	sprites.push_back(unique_ptr<TurnAnimation>(ta));
@@ -705,31 +768,72 @@ bool View::skipAnimatedCard(uint cid)
 	return false;
 }
 
-/** Either render score and errors in solo mode or both scores in two-player */
+/** Either render score and mistakes in solo mode or both scores in two-player */
 void View::renderPlayerInfo()
 {
 	string s;
-	if (game.numPlayers == 1) {
+	if (game.mode == GM_SURVIVOR) {
+		strprintf(s, _("Score: %d (%d Mistakes)"), game.players[0].score,
+						game.players[0].mistakes);
+		lblInfoLeft.setText(theme.fNormal, s);
+		strprintf(s, _("Stage: %d"), game.stage);
+		lblInfoRight.setText(theme.fNormal, s);
+	} else if (game.mode == GM_SOLO) {
 		strprintf(s, _("Matches: %d/%d"),
-				game.players[0].getScore(),
+				game.players[0].score,
 				game.numCards/config.matchsize);
-		lblScore.setText(theme.fNormal, s);
-		strprintf(s, _("Errors: %d"), game.players[0].getErrors());
-		lblErrors.setText(theme.fNormal, s);
-
+		lblInfoLeft.setText(theme.fNormal, s);
+		strprintf(s, _("Mistakes: %d"), game.players[0].mistakes);
+		lblInfoRight.setText(theme.fNormal, s);
 	} else {
-		/* two-player mode, use errors for second players score */
-		strprintf(s, _("%s: %d"),game.players[0].getName().c_str(),
-				game.players[0].getScore());
+		/* two-player mode */
+		strprintf(s, _("%s: %d"),game.players[0].name.c_str(),
+				game.players[0].score);
 		if (game.curPlayer == 0)
-			lblScore.setText(theme.fNormalHighlighted, s);
+			lblInfoLeft.setText(theme.fNormalHighlighted, s);
 		else
-			lblScore.setText(theme.fNormal, s);
-		strprintf(s, _("%s: %d"),game.players[1].getName().c_str(),
-				game.players[1].getScore());
+			lblInfoLeft.setText(theme.fNormal, s);
+		strprintf(s, _("%s: %d"),game.players[1].name.c_str(),
+				game.players[1].score);
 		if (game.curPlayer == 1)
-			lblErrors.setText(theme.fNormalHighlighted, s);
+			lblInfoRight.setText(theme.fNormalHighlighted, s);
 		else
-			lblErrors.setText(theme.fNormal, s);
+			lblInfoRight.setText(theme.fNormal, s);
 	}
 }
+
+/** Render a highlight frame for a flippable card. */
+void View::renderFlipFrame(uint cid)
+{
+	if (!config.autoflipframe)
+		return;
+
+	Card &c = game.cards[cid];
+	int bsize = 2; /* border size */
+	int blen = 0; /* length of a line */
+
+	if (autoFlipDelay.get() <= 0.30)
+		return;
+
+	/* scale 30%-90% of time to width which is also height since square */
+	if (autoFlipDelay.get() >= 0.9)
+		blen = c.w;
+	else
+		blen = (int)((autoFlipDelay.get()-0.3)*c.w/0.60);
+
+	SDL_SetRenderDrawColor(mrc, 255, 255, 255, autoFlipDelay.get()*255);
+	SDL_SetRenderDrawBlendMode(mrc, SDL_BLENDMODE_BLEND);
+
+	SDL_Rect top = {cxoff + c.x, cyoff + c.y - bsize, blen, bsize};
+	SDL_RenderDrawRect(renderer.get(), &top);
+	SDL_Rect bottom = {cxoff + c.x + c.w - blen, cyoff + c.y + c.h, blen, bsize};
+	SDL_RenderDrawRect(renderer.get(), &bottom);
+	SDL_Rect left = {cxoff + c.x - bsize, cyoff + c.y, bsize, blen};
+	SDL_RenderDrawRect(renderer.get(), &left);
+	SDL_Rect right = {cxoff + c.x + c.w, cyoff + c.y + c.h - blen, bsize, blen};
+	SDL_RenderDrawRect(renderer.get(), &right);
+
+
+
+	SDL_SetRenderDrawBlendMode(mrc, SDL_BLENDMODE_NONE);
+}
